%======================================================================
% A definition of the planning space for the blocks world
% 
% See Figure 16.2, p. 402 of
% "Prolog Programming for Artificial Intelligence", 2d Edition
%                        - by Ivan Bratko
%======================================================================

%----------------------------------------------------------------------
% Definition of action move(Block,From,To) in blocks world
%
% can(Action,Condition): Action possible if Condition true
%----------------------------------------------------------------------

can(move(Block,From,To),[clear(Block),clear(To),on(Block,From)]) :-
        block(Block),
        object(To),
        To \== Block,
        object(From),
        From \== To,
        Block \== From.

%----------------------------------------------------------------------
% add(Action,Relationships): Action establishes Relationships
%----------------------------------------------------------------------

add(move(Block,From,To),[on(Block,To),clear(From)]).

%----------------------------------------------------------------------
% del(Action,Relationships): Action destroys Relationships
%----------------------------------------------------------------------

del(move(Block,From,To),[on(Block,From),clear(To)]).

%----------------------------------------------------------------------
% An object is either a place or a block
%----------------------------------------------------------------------

object(X) :-
        place(X)
    ;
        block(X).

%----------------------------------------------------------------------
% Impossible( Goal, Goals): Goal is not possible in combination with Goals
%   This relation is only used by the "Goal Regression" planner.
%----------------------------------------------------------------------

impossible( on( X, X), _).                    % Block cannot be on itself

impossible( on( X, Y), Goals) :-
        member( clear( Y), Goals)
    ;
        member( on( X, Y1), Goals), Y1 \== Y  % Block cannot be in two places
    ;
        member( on( X1, Y), Goals), X1 \== X. % Two blocks cannot be in same
                                              %   place

impossible( clear( X), Goals) :-
        member( on( _, X), Goals).

%----------------------------------------------------------------------
% A blocks world
%----------------------------------------------------------------------

block(a).
block(b).
block(c).

place(1).
place(2).
place(3).
place(4).

%----------------------------------------------------------------------
% A state in blocks world
%----------------------------------------------------------------------
%
%  c
%  a   b
%  = = = =
%  1 2 3 4

state1( [clear(2), clear(4), clear(b), clear(c), on(a,1), on(b,3), on(c,a)]).


% The next predicate is necessary to run the planner 'planner-ME-GR-SP.pl'

start( [on(a,1), on(b,3), on(c,a), clear(b), clear(c), clear(2), clear(4)]).
