From:	CBS%UK.AC.OXFORD.MAIL-RELAY::COM.LOCKHEED.AUSTIN::REICH 14-SEP-1992 22:53:54.94
To:	POPX
CC:	
Subj:	planner-ME-GP.pl

Via: UK.AC.OXFORD.MAIL-RELAY; Mon, 14 Sep 92  22:53 BST
Received: from Shrike.Austin.Lockheed.COM by oxmail.ox.ac.uk with SMTP (PP) 
          id <29834-0@oxmail.ox.ac.uk>; Mon, 14 Sep 1992 22:48:39 +0100
Received: by shrike (4.1/1.49); Mon, 14 Sep 92 16:39:58 CDT
Message-Id: <9209142139.AA18368@shrike>
Received: by chisos.Austin.Lockheed.COM (15.11/15.6) id AA03883;
          Mon, 14 Sep 92 16:50:52 cdt
From: Al Reich <reich@COM.LOCKHEED.AUSTIN>
Subject: planner-ME-GP.pl
To: POPX@uk.ac.ox.vax
Date: Mon, 14 Sep 92 16:50:51 CDT
Cc: reich%chisos.AUSTIN.LOCKHEED.COM@COM.LOCKHEED.AUSTIN
In-Reply-To: <9209142048.AA17911@shrike>; from "Jocelyn Paine" at Sep 14, 92 10:01 pm
Mailer: Elm [revision: 64.9]

%======================================================================
% MODULE: planner-ME-GP.p
%
% A simple means-ends planner with goal protection.
% 
% See Figure 16.6, p. 411 of
% "Prolog Programming for Artificial Intelligence", 2d Edition
%                        - by Ivan Bratko
%======================================================================

%----------------------------------------------------------------------
% plan( InitialState, Goals, ProtectedGoals, Plan, FinalState):
%   Goals true in FinalState, ProtectedGoals never destroyed by Plan
%----------------------------------------------------------------------

plan( InitialState, Goals, Plan, FinalState) :-
        plan( InitialState, Goals, [], Plan, FinalState).

plan( State, Goals, _, [], State) :-
        satisfied( State, Goals).          % Goals true in State

% The way plan is decomposed into stages by conc, the precondition plan
% (PrePlan) is found in breadth-first fashion.  However, the length of
% the rest of plan is not restricted and goals are achieved in
% depth-first style.

plan( State, Goals, Protected, Plan, FinalState) :-
        conc( Plan, _, _),                 % Use breath first search
        conc( PrePlan, [Action | PostPlan], Plan),  % Divide plan
        select( State, Goals, Goal),                % Select a goal
        achieves( Action, Goal),
        can( Action, Condition),
        preserves( Action, Protected),
        plan( State, Condition, Protected, PrePlan, MidState1),
        apply( MidState1, Action, MidState2),
        plan( MidState2, Goals, [Goal | Protected], PostPlan, FinalState).

%----------------------------------------------------------------------
% satisfied( State, Goals): Goals are true in State
%----------------------------------------------------------------------

satisfied( State, []).

satisfied( State, [Goal | Goals]) :-
        member( Goal, State),                        % Usual list membership
        satisfied( State, Goals).

select( State, Goals, Goal) :-
        member( Goal, Goals),
        not member( Goal, State).

%----------------------------------------------------------------------
% achieves( Action, Goal): Goal is in the add-list of Action
%----------------------------------------------------------------------

achieves( Action, Goal) :-
        add( Action, Goals),
        member( Goal, Goals).

%----------------------------------------------------------------------
% preserves( Action, Goals):
%   Action does not destroy any one of Goals
%----------------------------------------------------------------------

preserves( Action, Goals) :-
        del( Action, Relations),
        not ( member( Goal, Relations),
              member( Goal, Goals)).

%----------------------------------------------------------------------
% apply( State, Action, NewState):
%   Action executed in State produces NewState
%----------------------------------------------------------------------

apply( State, Action, NewState) :-
        del( Action, DelList),
        delete( State, DelList, State1),!,
        add( Action, AddList),
        conc( AddList, State1, NewState).

%----------------------------------------------------------------------
%                             EOF
%----------------------------------------------------------------------
