From:	CBS%UK.AC.OXFORD.MAIL-RELAY::COM.LOCKHEED.AUSTIN::REICH 14-SEP-1992 22:58:54.56
To:	POPX
CC:	
Subj:	planner-ME.pl

Via: UK.AC.OXFORD.MAIL-RELAY; Mon, 14 Sep 92  22:58 BST
Received: from Shrike.Austin.Lockheed.COM by oxmail.ox.ac.uk with SMTP (PP) 
          id <29889-0@oxmail.ox.ac.uk>; Mon, 14 Sep 1992 22:53:38 +0100
Received: by shrike (4.1/1.49); Mon, 14 Sep 92 16:44:58 CDT
Message-Id: <9209142144.AA18406@shrike>
Received: by chisos.Austin.Lockheed.COM (15.11/15.6) id AA03934;
          Mon, 14 Sep 92 16:55:53 cdt
From: Al Reich <reich@COM.LOCKHEED.AUSTIN>
Subject: planner-ME.pl
To: POPX@uk.ac.ox.vax
Date: Mon, 14 Sep 92 16:55:52 CDT
Cc: reich%chisos.AUSTIN.LOCKHEED.COM@COM.LOCKHEED.AUSTIN
In-Reply-To: <9209142048.AA17911@shrike>; from "Jocelyn Paine" at Sep 14, 92 10:01 pm
Mailer: Elm [revision: 64.9]

%======================================================================
% MODULE: planner-ME.p
%
% A simple means-ends planner.
% 
% See Figure 16.5, p. 406 of
% "Prolog Programming for Artificial Intelligence", 2d Edition
%                        - by Ivan Bratko
% 
% NOTE:  This planner uses the 'add', 'del', and 'can' relations that
%        were originally introduced in the STRIPS planner.
%        See "STRIPS: A New Approach to the Application of Theorem
%        Proving to Problem Solving.", by R.E. Fikes and N.J. Nilsson,
%        in Artificial Intelligence 2: 189-208.
%======================================================================

%----------------------------------------------------------------------
% plan( InitialState, Goals, Plan, FinalState)
%----------------------------------------------------------------------

plan( State, Goals, [], State) :-
        satisfied( State, Goals).          % Goals true in State

% The way plan is decomposed into stages by conc, the precondition plan
% (PrePlan) is found in breadth-first fashion.  However, the length of
% the rest of plan is not restricted and goals are achieved in
% depth-first style.

plan( State, Goals, Plan, FinalState) :-
        conc( Plan, _, _),                 % Use breath first search
        conc( PrePlan, [ Action | PostPlan], Plan), % Divide plan
        select( State, Goals, Goal),                % Select a goal
        achieves( Action, Goal),
        can( Action, Condition),
        plan( State, Condition, PrePlan, MidState1),
        apply( MidState1, Action, MidState2),
        plan( MidState2, Goals, PostPlan, FinalState).

%----------------------------------------------------------------------
% satisfied( State, Goals): Goals are true in State
%----------------------------------------------------------------------

satisfied( State, []).

satisfied( State, [Goal | Goals]) :-
        member( Goal, State),                        % Usual list membership
        satisfied( State, Goals).

select( State, Goals, Goal) :-
        member( Goal, Goals),
        not member( Goal, State).

%----------------------------------------------------------------------
% achieves( Action, Goal): Goal is in the add-list of Action
%----------------------------------------------------------------------

achieves( Action, Goal) :-
        add( Action, Goals),
        member( Goal, Goals).

%----------------------------------------------------------------------
% apply( State, Action, NewState):
%   Action executed in State produces NewState
%----------------------------------------------------------------------

apply( State, Action, NewState) :-
        del( Action, DelList),
        delete( State, DelList, State1),!,
        add( Action, AddList),
        conc( AddList, State1, NewState).

%----------------------------------------------------------------------
% delete( L1, L2, Diff): Diff is the set-difference of lists L1 and L2
%----------------------------------------------------------------------

delete( [], _, []).

delete( [X | L1], L2, Diff) :-
        member( X, L2),!,
        delete( L1, L2, Diff).

delete( [X | L1], L2, [X | Diff]) :-
        delete( L1, L2, Diff).

%----------------------------------------------------------------------
%                             EOF
%----------------------------------------------------------------------
